@@ -9,8 +9,10 @@
  *
  * Copyright (c) 2012      Oracle and/or its affiliates.  All rights reserved.
  * Copyright (c) 2014      NVIDIA Corporation.  All rights reserved.
- * Copyright (c) 2015      Research Organization for Information Science
+ * Copyright (c) 2015-2016 Research Organization for Information Science
  *                         and Technology (RIST). All rights reserved.
+ * Copyright (c) 2015      Los Alamos National Security, LLC. All rights
+ *                         reserved.
  *
  */
 #ifndef __NBC_INTERNAL_H__
@@ -30,6 +32,7 @@
 #include "ompi/include/ompi/constants.h"
 #include "ompi/request/request.h"
 #include "ompi/datatype/ompi_datatype.h"
+#include "ompi/communicator/communicator.h"
 
 #include <stdlib.h>
 #include <stdio.h>
@@ -87,6 +90,7 @@ typedef struct {
   int count;
   MPI_Datatype datatype;
   int dest;
+  bool local;
 } NBC_Args_send;
 
 /* the receive argument struct */
@@ -96,6 +100,7 @@ typedef struct {
   int count;
   MPI_Datatype datatype;
   int source;
+  bool local;
 } NBC_Args_recv;
 
 /* the operation argument struct */
@@ -104,11 +109,9 @@ typedef struct {
   char tmpbuf1;
   void *buf2;
   char tmpbuf2;
-  void *buf3;
-  char tmpbuf3;
-  int count;
   MPI_Op op;
   MPI_Datatype datatype;
+  int count;
 } NBC_Args_op;
 
 /* the copy argument struct */
@@ -136,8 +139,11 @@ typedef struct {
 /* internal function prototypes */
 int NBC_Sched_create(NBC_Schedule* schedule);
 int NBC_Sched_send(void* buf, char tmpbuf, int count, MPI_Datatype datatype, int dest, NBC_Schedule *schedule);
+int NBC_Sched_local_send (void* buf, char tmpbuf, int count, MPI_Datatype datatype, int dest,NBC_Schedule *schedule);
 int NBC_Sched_recv(void* buf, char tmpbuf, int count, MPI_Datatype datatype, int source, NBC_Schedule *schedule);
-int NBC_Sched_op(void* buf3, char tmpbuf3, void* buf1, char tmpbuf1, void* buf2, char tmpbuf2, int count, MPI_Datatype datatype, MPI_Op op, NBC_Schedule *schedule);
+int NBC_Sched_local_recv (void* buf, char tmpbuf, int count, MPI_Datatype datatype, int source, NBC_Schedule *schedule);
+int NBC_Sched_op (void* buf1, char tmpbuf1, void* buf2, char tmpbuf2, int count, MPI_Datatype datatype,
+                  MPI_Op op, NBC_Schedule *schedule);
 int NBC_Sched_copy(void *src, char tmpsrc, int srccount, MPI_Datatype srctype, void *tgt, char tmptgt, int tgtcount, MPI_Datatype tgttype, NBC_Schedule *schedule);
 int NBC_Sched_unpack(void *inbuf, char tmpinbuf, int count, MPI_Datatype datatype, void *outbuf, char tmpoutbuf, NBC_Schedule *schedule);
 int NBC_Sched_barrier(NBC_Schedule *schedule);
@@ -503,9 +509,13 @@ static inline int NBC_Copy(void *src, int srccount, MPI_Datatype srctype, void *
   } else {
     /* we have to pack and unpack */
     res = MPI_Pack_size(srccount, srctype, comm, &size);
-    if (MPI_SUCCESS != res) { printf("MPI Error in MPI_Pack_size() (%i:%i)\n", res, size); return (MPI_SUCCESS == res) ? MPI_ERR_SIZE : res; }
+    if (MPI_SUCCESS != res) {
+      printf ("MPI Error in MPI_Pack_size() (%i:%i)", res, size);
+      return res;
+    }
+
     if (0 == size) {
-        return NBC_OK;
+        return OMPI_SUCCESS;
     }
     packbuf = malloc(size);
     if (NULL == packbuf) { printf("Error in malloc()\n"); return res; }
