@@ -10,7 +10,9 @@
  * Copyright (c) 2004-2006 The Regents of the University of California.
  *                         All rights reserved.
  * Copyright (c) 2011-2015 NVIDIA Corporation.  All rights reserved.
- * Copyright (c) 2015 Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2015      Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2016      Research Organization for Information Science
+ *                         and Technology (RIST). All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -252,7 +254,7 @@ static int mca_common_cuda_init(opal_common_cuda_function_table_t *ftable)
 int mca_common_cuda_stage_one_init(void)
 {
     int retval, i, j;
-    char *cudalibs[] = {"libcuda.so.1", NULL};
+    char *cudalibs[] = {"libcuda.so.1", "libcuda.dylib", NULL};
     char *searchpaths[] = {"", "/usr/lib64", NULL};
     char **errmsgs = NULL;
     char *errmsg = NULL;
@@ -317,7 +319,7 @@ int mca_common_cuda_stage_one_init(void)
 #endif /* OPAL_CUDA_SUPPORT_41 */
 
     /* Use this flag to test cuMemcpyAsync vs cuMemcpy */
-    mca_common_cuda_cumemcpy_async = 0;
+    mca_common_cuda_cumemcpy_async = 1;
     (void) mca_base_var_register("ompi", "mpi", "common_cuda", "cumemcpy_async",
                                  "Set to 0 to force CUDA cuMemcpy instead of cuMemcpyAsync/cuStreamSynchronize",
                                  MCA_BASE_VAR_TYPE_INT, NULL, 0, 0,
