@@ -12,8 +12,9 @@
  *                         reserved.
  * Copyright (c) 2010      Cisco Systems, Inc.  All rights reserved.
  * Copyright (c) 2012-2013 Sandia National Laboratories.  All rights reserved.
- * Copyright (c) 2015      Research Organization for Information Science
+ * Copyright (c) 2015-2016 Research Organization for Information Science
  *                         and Technology (RIST). All rights reserved.
+ * Copyright (c) 2016      FUJITSU LIMITED.  All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -390,7 +391,7 @@ static inline int ompi_osc_pt2pt_put_w_req (const void *origin_addr, int origin_
             OBJ_RETAIN(target_dt);
 
             ret = ompi_osc_pt2pt_isend_w_cb ((void *) packed_ddt, ddt_len, MPI_BYTE, target,
-                                            tag, module->comm, ompi_osc_pt2pt_dt_send_complete,
+                                            tag_to_target(tag), module->comm, ompi_osc_pt2pt_dt_send_complete,
                                             target_dt);
             if (OPAL_UNLIKELY(OMPI_SUCCESS != ret)) {
                 break;
@@ -426,10 +427,10 @@ static inline int ompi_osc_pt2pt_put_w_req (const void *origin_addr, int origin_
             if (request) {
                 request->outstanding_requests = 1;
                 ret = ompi_osc_pt2pt_isend_w_cb (origin_addr, origin_count, origin_dt,
-                                                target, tag, module->comm, ompi_osc_pt2pt_req_comm_complete,
+                                                target, tag_to_target(tag), module->comm, ompi_osc_pt2pt_req_comm_complete,
                                                 request);
             } else {
-                ret = ompi_osc_pt2pt_component_isend (module,origin_addr, origin_count, origin_dt, target, tag,
+                ret = ompi_osc_pt2pt_component_isend (module,origin_addr, origin_count, origin_dt, target, tag_to_target(tag),
                                                      module->comm);
             }
         }
@@ -568,7 +569,7 @@ ompi_osc_pt2pt_accumulate_w_req (const void *origin_addr, int origin_count,
             OBJ_RETAIN(target_dt);
 
             ret = ompi_osc_pt2pt_isend_w_cb ((void *) packed_ddt, ddt_len, MPI_BYTE, target,
-                                            tag, module->comm, ompi_osc_pt2pt_dt_send_complete,
+                                            tag_to_target(tag), module->comm, ompi_osc_pt2pt_dt_send_complete,
                                             target_dt);
             if (OPAL_UNLIKELY(OMPI_SUCCESS != ret)) {
                 break;
@@ -607,10 +608,10 @@ ompi_osc_pt2pt_accumulate_w_req (const void *origin_addr, int origin_count,
             if (request) {
                 request->outstanding_requests = 1;
                 ret = ompi_osc_pt2pt_isend_w_cb (origin_addr, origin_count, origin_dt,
-                                                target, tag, module->comm, ompi_osc_pt2pt_req_comm_complete,
+                                                target, tag_to_target(tag), module->comm, ompi_osc_pt2pt_req_comm_complete,
                                                 request);
             } else {
-                ret = ompi_osc_pt2pt_component_isend (module, origin_addr, origin_count, origin_dt, target, tag,
+                ret = ompi_osc_pt2pt_component_isend (module, origin_addr, origin_count, origin_dt, target, tag_to_target(tag),
                                                      module->comm);
             }
         }
@@ -725,7 +726,7 @@ int ompi_osc_pt2pt_compare_and_swap (const void *origin_addr, const void *compar
     osc_pt2pt_copy_for_send (ptr, dt->super.size, compare_addr, proc, 1, dt);
 
     request->outstanding_requests = 1;
-    ret = ompi_osc_pt2pt_irecv_w_cb (result_addr, 1, dt, target, tag, module->comm,
+    ret = ompi_osc_pt2pt_irecv_w_cb (result_addr, 1, dt, target, tag_to_origin(tag), module->comm,
                                     NULL, ompi_osc_pt2pt_req_comm_complete, request);
     if (OPAL_UNLIKELY(OMPI_SUCCESS != ret)) {
         return ret;
@@ -901,7 +902,7 @@ static inline int ompi_osc_pt2pt_rget_internal (void *origin_addr, int origin_co
             OBJ_RETAIN(target_dt);
 
             ret = ompi_osc_pt2pt_isend_w_cb ((void *) packed_ddt, ddt_len, MPI_BYTE, target,
-                                            tag, module->comm, ompi_osc_pt2pt_dt_send_complete,
+                                            tag_to_target(tag), module->comm, ompi_osc_pt2pt_dt_send_complete,
                                             target_dt);
             if (OPAL_UNLIKELY(OMPI_SUCCESS != ret)) {
                 break;
@@ -916,7 +917,7 @@ static inline int ompi_osc_pt2pt_rget_internal (void *origin_addr, int origin_co
 
         /* TODO -- store the request somewhere so we can cancel it on error */
         pt2pt_request->outstanding_requests = 1;
-        ret = ompi_osc_pt2pt_irecv_w_cb (origin_addr, origin_count, origin_dt, target, tag,
+        ret = ompi_osc_pt2pt_irecv_w_cb (origin_addr, origin_count, origin_dt, target, tag_to_origin(tag),
                                         module->comm, NULL, ompi_osc_pt2pt_req_comm_complete, pt2pt_request);
     } while (0);
 
@@ -1137,7 +1138,7 @@ int ompi_osc_pt2pt_rget_accumulate_internal (const void *origin_addr, int origin
             OBJ_RETAIN(target_datatype);
 
             ret = ompi_osc_pt2pt_isend_w_cb ((void *) packed_ddt, ddt_len, MPI_BYTE, target_rank,
-                                            tag, module->comm, ompi_osc_pt2pt_dt_send_complete,
+                                            tag_to_target(tag), module->comm, ompi_osc_pt2pt_dt_send_complete,
                                             target_datatype);
             if (OPAL_UNLIKELY(OMPI_SUCCESS != ret)) {
                 break;
@@ -1150,7 +1151,7 @@ int ompi_osc_pt2pt_rget_accumulate_internal (const void *origin_addr, int origin
             ptr += ddt_len;
         }
 
-        ret = ompi_osc_pt2pt_irecv_w_cb (result_addr, result_count, result_datatype, target_rank, tag,
+        ret = ompi_osc_pt2pt_irecv_w_cb (result_addr, result_count, result_datatype, target_rank, tag_to_origin(tag),
                                         module->comm, NULL, ompi_osc_pt2pt_req_comm_complete, pt2pt_request);
         if (OPAL_UNLIKELY(OMPI_SUCCESS != ret)) {
             break;
@@ -1169,7 +1170,7 @@ int ompi_osc_pt2pt_rget_accumulate_internal (const void *origin_addr, int origin
             osc_pt2pt_hton(header, proc);
 
             ret = ompi_osc_pt2pt_isend_w_cb (origin_addr, origin_count, origin_datatype, target_rank,
-                                            tag, module->comm, ompi_osc_pt2pt_req_comm_complete, pt2pt_request);
+                                            tag_to_target(tag), module->comm, ompi_osc_pt2pt_req_comm_complete, pt2pt_request);
         }
     } while (0);
 
