@@ -29,9 +29,12 @@
 #define ompi_add_error_code_f pompi_add_error_code_f
 #define ompi_add_error_string_f pompi_add_error_string_f
 #define ompi_address_f pompi_address_f
+#define ompi_aint_add_f pompi_aint_add_f
+#define ompi_aint_diff_f pompi_aint_diff_f
 #define ompi_allgather_f pompi_allgather_f
 #define ompi_allgatherv_f pompi_allgatherv_f
 #define ompi_alloc_mem_f pompi_alloc_mem_f
+#define ompi_alloc_mem_cptr_f pompi_alloc_mem_cptr_f
 #define ompi_allreduce_f pompi_allreduce_f
 #define ompi_alltoall_f pompi_alltoall_f
 #define ompi_alltoallv_f pompi_alltoallv_f
@@ -348,12 +351,15 @@
 #define ompi_win_allocate_cptr_f pompi_win_allocate_cptr_f
 #define ompi_win_allocate_shared_f pompi_win_allocate_shared_f
 #define ompi_win_allocate_shared_cptr_f pompi_win_allocate_shared_cptr_f
+#define ompi_win_attach_f pompi_win_attach_f
 #define ompi_win_call_errhandler_f pompi_win_call_errhandler_f
 #define ompi_win_complete_f pompi_win_complete_f
 #define ompi_win_create_f pompi_win_create_f
+#define ompi_win_create_dynamic_f pompi_win_create_dynamic_f
 #define ompi_win_create_errhandler_f pompi_win_create_errhandler_f
 #define ompi_win_create_keyval_f pompi_win_create_keyval_f
 #define ompi_win_delete_attr_f pompi_win_delete_attr_f
+#define ompi_win_detach_f pompi_win_detach_f
 #define ompi_win_fence_f pompi_win_fence_f
 #define ompi_win_flush_f pompi_win_flush_f
 #define ompi_win_flush_all_f pompi_win_flush_all_f
